<?php
class SynoDLMSearchRapideo {

    private $api_url = 'https://www.rapideo.pl/api/rest';
    private $authtoken_file = "/tmp/rapideo_authtoken.txt";
    private $log_file = "/tmp/rapideo_logfile.txt";
    private $account_endpoint = 'account';
    private $search_endpoint = 'search';
    private $update_endpoint = 'update';

    public function __construct() {
        $this->addon_info_path = str_replace('userplugins/Rapideo', 'userhosts/RapideoOfficial/INFO',  dirname(__file__));
        $this->search_query_url = $this->api_url . '/' . $this->search_endpoint;
    }

    private function api_request($endpoint = "", $method = 'GET' , $data = array(), $conv_json = false, $plugin = NULL) {

        if($conv_json) {
            $json_data = json_encode($data, JSON_PRETTY_PRINT);
            unset($data);
            $data = array($json_data);
        }

        try {
            if(preg_match('/http:\/\//i', $endpoint) || preg_match('/https:\/\//i', $endpoint)) {
                $curl = curl_init($endpoint);
            } else {
                $curl = curl_init($this->api_url .'/' . $endpoint);
            }
            curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($curl, CURLOPT_TIMEOUT, 10);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER , true);
            if(strtolower($method) == 'post') {
                curl_setopt($curl, CURLOPT_POST, true);
                if(!empty($data)) {
                    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
                }
            }
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('User-Agent: Synology'));
            $response = curl_exec($curl);
            curl_close($curl);

            if($response) {
                $response = json_decode( $response , true);

                if(isset($response['error']) && $response['error'] == 0) {
                    if($plugin) {
                        $this->displayConsole($plugin,$this->getError($response['error']));
                    }
                }

                if(!empty($response['error'])) {
                    $this->save_log($response['message'] , $response['error']);
                    $this->reset_authtoken();
                }
            }
        } catch(Exception $e) {
            $this->save_log($e->getMessage());
            $this->send_log($e->getMessage());
        }

        return array('response'=>$response);

    }

    private function reset_authtoken() {
        if(file_exists($this->authtoken_file)) {
            unlink($this->authtoken_file);
        }
        return true;
    }

    private function load_authtoken() {
        if(!file_exists($this->authtoken_file)) return;
        if(filesize($this->authtoken_file) == 0) return;

        $handle = fopen($this->authtoken_file , 'r');
        $authtoken = fread($handle, filesize($this->authtoken_file));
        fclose($handle);
        if(!empty($authtoken)) return $authtoken;
    }


    public function prepare($curl, $keyword) {
        $this->_curl_setup($curl, $this->search_query_url, 'post',  array('authtoken'=>$this->load_authtoken(),'keyword'=>$keyword , 'mode'=>'synology', 'display'=>50));
    }

    public function parse($plugin, $response) {
        if($response) {
            $response = json_decode($response, true);
            if(isset($response['error']) && !empty($response['message'])) {
                $this->displayConsole($plugin, $this->getError($response['error']));
            } else if(isset($response['console'])) {
                $this->Console($plugin, $response['console']);
            } else {
                $this->processResultList($plugin, $response);
            }
        }
    }

    private function getError($errno = 0) {
        switch($errno) {
            case "0":
                return "Zaloguj się w ustawieniach wtyczki";
                break;
            default:
                return "Wystąpił błąd";
                break;
        }
    }

    private function _curl_setup($curl, $url = "" , $mode = "",  $data = array())
    {
        curl_setopt($curl, CURLOPT_URL, $url);
        if(strtolower($mode) == 'post') {
            curl_setopt($curl, CURLOPT_POST, true);
            if(!empty($data)) {
                curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
            }
        }
        curl_setopt($curl, CURLOPT_FAILONERROR, 1);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER,1);
        curl_setopt($curl, CURLOPT_TIMEOUT, 10);
        curl_setopt($curl, CURLOPT_USERAGENT, 'SYNOLOGY');
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
    }

    private function processResultList($plugin, $response)
    {
        if(!empty($response['search']['search_result'])) {
            foreach ($response['search']['search_result'] as $result) {

                $parse_url = parse_url($result['url']);

                $recordToAdd = array();
                $recordToAdd['title'] = $result['filename_long'];
                $recordToAdd['download'] = rawurldecode($result['url_encoded']);
                $recordToAdd['size'] = (int)$result['filesize_bytes'];
                $recordToAdd['page'] = $parse_url['scheme']."://" . $parse_url['host'];
                $recordToAdd["seeds"] = (int)$result['best'];
                $recordToAdd["leechs"] = 0;
                if(!empty($result['movie'])) {
                    $recordToAdd["category"] = 'video';
                } else {
                    $recordToAdd["category"] = 'other';
                }

                $plugin->addResult(
                    $recordToAdd['title'],
                    $recordToAdd['download'],
                    $recordToAdd['size'], date('Y-m-d', time()),
                    $recordToAdd['page'],
                    "",
                    $recordToAdd["seeds"] ,
                    $recordToAdd["leechs"],
                    $recordToAdd["category"]
                );
            }
        }
    }

    private function save_log($message = "" , $error = null) {
        if(is_array($message)) return;

        $date = date("Y-m-d H:i:s" , time());

        try {
            clearstatcache();
            if(file_exists($this->log_file) && filesize($this->log_file) > (1*1024*1024)) {
                unlink($this->log_file);
            }

            $log['message'] = $message;
            if(!empty($error)) $log['message_type'] = 'error';
            else $log['message_type'] = 'notice';

            $log['logtime'] = $date;


            $handle = @fopen($this->log_file , 'a');
            @fwrite($handle,  json_encode($log));
            @fwrite($handle, "\r\n\r\n");
            @fclose($handle);
        } catch(Exception $e) {
            $fp = @fopen($this->log_file , 'a');
            @fwrite($fp, json_encode(array('message'=>$e->getMessage() , 'logtime'=>$date)));
            @fwrite($fp, "\r\n\r\n");
            fclose($fp);
        }
    }

    private function send_log($log = "") {
        $this->api_request($this->log_endpoint , 'post', array('log'=>$log, 'authtoken'=>$this->load_authtoken())  , true);
    }


    private function displayConsole(&$plugin ,$title = "" , $logtime = "", $category = "", $download_url = "") {
        if(empty($logtime)) $logtime = date('Y-m-d', time());
        if(empty($category)) $category = 'Console';
        if(empty($download_url)) $download_url = '';

        $plugin->addResult(
            $title,
            $download_url,
            '',
            $logtime,
            0,
            "",
            0,
            0,
            $category
        );
    }

    private function Console(&$plugin , $option = "") {
        switch($option) {
            case "console::help":
                return $this->ConsoleMenu($plugin);
                break;
            case "console::account":
                return $this->accountInfo($plugin);
                break;
            case "console::logs":
                return $this->Logs($plugin);
                break;
            case "console::logs_clear" :
                return $this->LogsClear($plugin);
                break;
            case "console::hosters":
                return $this->displayHosters($plugin);
                break;
            case "console::version" :
                $current_version = $this->CheckVersion();
                return $this->displayConsole($plugin, 'Wersja wtyczki: ' . $current_version);
                break;
            case "console::update" :
                return $this->Update($plugin);
                break;
            default:
                return $this->displayConsole($plugin,'Nieprawidłowa opcja konsoli');
                break;
        }
    }

    private function ConsoleMenu(&$plugin) {
        $commands = array();
        $options = array(
            'console::help'=> 'Pomoc',
            'console::account'=> 'Informacja o koncie',
            'console::logs'=> 'Wyświetl logi',
            'console::logs_clear'=> 'Skasuj logi',
            'console::hosters'=> 'Obsługiwane serwisy hostingowe',
            'console::version'=> 'Wersja wtyczki',
            'console::update'=> 'Aktualizacja wtyczki',
        );

        foreach($options as $command => $value) {
            $this->displayConsole($plugin,$command .' - ' . $value);
        }
    }

    private function accountInfo(&$plugin) {
        $account = array();
        $accountInfo = $this->api_request($this->account_endpoint , 'post' , array('authtoken'=>$this->load_authtoken()) , false, $plugin);
        if(!empty($accountInfo['response']['account'])) {
            $login = isset($accountInfo['response']['account']['login']) ? $accountInfo['response']['account']['login'] : '';
            $email = isset($accountInfo['response']['account']['email']) ? $accountInfo['response']['account']['email'] : '';
            $transfer_left_gb = isset($accountInfo['response']['account']['transfer_left_gb']) ? $accountInfo['response']['account']['transfer_left_gb'] : 0;
            $premium_active = isset($accountInfo['response']['account']['premium_active']) ? $accountInfo['response']['account']['premium_active'] : 0;
            $premium_expire_date = isset($accountInfo['response']['account']['premium_expire_date']) ? $accountInfo['response']['account']['premium_expire_date'] : '';
            $premium_left_gb = isset($accountInfo['response']['account']['premium_left_gb']) ? $accountInfo['response']['account']['premium_left_gb'] : 0;

            $this->displayConsole($plugin,'Login: ' . $login);
            $this->displayConsole($plugin,'E-mail: ' . $email);
            if(!empty($premium_active)) {
                $this->displayConsole($plugin,'Ważność konta Premium: ' . $premium_expire_date);
                $this->displayConsole($plugin,'Transfer konta Premium: ' . $premium_left_gb . ' GB');
                $this->displayConsole($plugin,'Transfer dodatkowy: ' . $transfer_left_gb . ' GB');
            } else {
                $this->displayConsole($plugin,'Transfer standardowy: ' . $transfer_left_gb . ' GB');
            }
            return $account;
        }
    }

    private function Logs(&$plugin) {
        if(!file_exists($this->log_file)) {
            try {
                $this->displayConsole($plugin, 'Brak logów');
            } catch(Exception $e) {
                $this->save_log($e->getMessage());
            }
            return true;
        }

        $found = array();

        $handle = @fopen($this->log_file, 'r');
        $readlogs = @fread($handle, filesize($this->log_file));
        @fclose($handle);

        $logs = explode("\r\n\r\n", $readlogs);

        foreach($logs as $log) {
            if(empty($log)) continue;

            try {
                $decode_log = json_decode($log, true);
                if(isset($decode_log['message_type']) && $decode_log['message_type'] == 'error') {
                    $message = "Błąd: " . $decode_log['message'];
                } else {
                    $message = $decode_log['message'];
                }
                $this->displayConsole($plugin, $message, $decode_log['logtime'], 'log');
            } catch(Exception $e) {
                $this->save_log($e->getMessage());
            }
        }

        return true;
    }

    private function LogsClear(&$plugin) {
        if(file_exists($this->log_file)) {
            unlink($this->log_file);
        }
        return $this->Logs($plugin);
    }

    private function GetHosters() {
        $hosters = array('Brak obsługiwanych hostingów');
        if(file_exists($this->addon_info_path)) {
            $addonInfo = json_decode(file_get_contents($this->addon_info_path), true);
            if(empty($addonInfo['hostprefix'])) return $hosters;
            $hosters = explode(',' , $addonInfo['hostprefix']);
            return $hosters;
        }
    }

    private function displayHosters(&$plugin) {
        $hosters =  array();
        $gethosters = $this->GetHosters();
        foreach($gethosters as $hoster) {
            $this->displayConsole($plugin, $hoster);
        }
    }

    private function CheckVersion() {
        if(file_exists($this->addon_info_path)) {
            $addon_json = json_decode(file_get_contents($this->addon_info_path) , true);
            return (int)str_replace("." , "", $addon_json['version']);
        }
    }

    private function Update(&$plugin) {
        $current_version = (int)$this->CheckVersion();
        $fetch_update = $this->api_request($this->update_endpoint , 'post' , array('device'=>'synology'));
        $remote_version = isset($fetch_update['response']['version']) ? (int)$fetch_update['response']['version'] : 0;
        $remote_url = isset($fetch_update['response']['url']) ? $fetch_update['response']['url'] : '';
        $remote_size = isset($fetch_update['response']['size']) ? $fetch_update['response']['size'] : 0;

        if(empty($current_version) || empty($remote_version) || empty($remote_url))   {
            return $this->displayConsole($plugin, 'Nie udało się sprawdzić aktualizacji - ' . $remote_version);
        }

        if(is_numeric($current_version) && is_numeric($remote_version)) {
            if($current_version == $remote_version) {
                return $this->displayConsole($plugin,'Brak dostępnej aktualizacji (wersja wtyczki: ' . $current_version.', wersja aktualna: ' . $remote_version.')');
            } else if($remote_version  > $current_version) {
                $this->displayConsole($plugin, 'Nowa aktualizacja jest dostępna (wersja ' . $remote_version.')');
                $this->displayConsole($plugin, 'Kliknij dwa razy tutaj aby pobrać', "", "" , $remote_url);
                $this->displayConsole($plugin, 'Następnie pobierz ją na kompter z aplikacji File Station -> ');
                $this->displayConsole($plugin, '-> oraz dodaj do aplikacji Download Station');
                $this->displayConsole($plugin, 'Instrukcja instalacji wtyczki pod adresem ->');
                $this->displayConsole($plugin, '-> https://www.rapideo.pl/plugins/synology');
            }
        }
    }
}
?>